/* ============================================================================
 * File Name: DelSig_lock-in
 *
 * Description:
 *   Lock-in detector using DelSig_ADC and digital Filter.
 * 
 *   Tunable frequency and phase are generated using 24-bit direct digital
 *   synthesis custom component DDS24.  
 *
 *   Rotary encoder is used to tune output frequency of the DDS generator between
 *   100 kHz and 300 kHz. Secondary DDS output provides synchronous reference with
 *   phase shift tunable from 0 deg to 360 deg. Pressing the button switches between
 *   frequency and phase. The Decoder is set to operate in unbounded integer mode,
 *   while all necessary calculations for frequency and phase are being performed
 *   in the main code using only direction of rotation. This provides somewhat
 *   greater flexibility in expense for extra custom code. 
 *
 *   Averaged output transmitted to Multichart plotting GUI using UART-USB
 *   convertor built into CY8KIT-059 KitProg programmer.
 *
 *   ADC data averaging is demonstrated using 3 methods:
 *   (a) direct averaging ADC data using IIR exponential filter
 *   (b) FIR with digital Filter using DMA transfer from ADC
 *   (c) IIR averaging of data passed through FIR Filter block and decimated.
 *
 * ============================================================================
 * PROVIDED AS-IS, NO WARRANTY OF ANY KIND, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * FREE TO SHARE, USE AND MODIFY UNDER TERMS: CREATIVE COMMONS - SHARE ALIKE
 * ============================================================================
*/



#include <project.h>
#include <DMA.h>
#include <Multichart.h>

#define BlinkLED1(); {Pin_LED1_Write(1); CyDelayUs(10); Pin_LED1_Write(0);} // blink LED indicator

//===========================================
// ISR report (to output data via UART) 
//===========================================
static volatile CYBIT isrReport_flag = 0; // semaphore flag 
CY_ISR(IRQ_InterruptReport)
{
    isrReport_flag = 1;
}

//===========================================
// Interrupt generated on ADC data sample ready 
//===========================================
static volatile uint32 ADC1_result;    // ADC result
static volatile CYBIT isrADC_flag = 0; // semaphore flag
CY_ISR(ADC1_IRQ_myInterrupt)
{
    isrADC_flag = 1;
    ADC1_result = ADC1_GetResult32();
}

//===========================================
// Interrupt generated on Filter sample-ready 
//===========================================
static volatile int16 FILTER_result;
static volatile CYBIT isrDFB_flag = 0;
static volatile uint8 dec_counter = 31;
CY_ISR(DFB_DataReady)
{
    //Filter_ClearInterruptSource(); 
    FILTER_result=Filter_Read16(Filter_CHANNEL_A); // also clears interrupt
    
    if (dec_counter-- != 0) return;
    dec_counter = 31; // reset
    
    isrDFB_flag = 1;
}


//===========================================
// Global variables
//===========================================

    #define ENCODE_FREQ  1u
    #define ENCODE_PHASE 0u 
    uint8   Encode_Type = ENCODE_FREQ;

    double Freq;            // DDS set frequency 
    uint8  Phase;           // DDS set phase 

    int32  ADC1_aver;       // ADC value filtered thru IIR 
    int32  IIRout;          // value filtered thru IIR 
    



//===========================================
// Function prototypes
//===========================================
void UpdateDDSfrequency();
void UpdateDDSphase();
int32 IIR_Filter(int32 input);
int32 IIR_Filter2(int32 input);    

void Initialize()
{   
    CyGlobalIntEnable;             //enable global interrupts.
     
    UART_1_Start();//hardware UART (Transmit only)
    CyDelay(100);
    
    Decoder_1_Start();
    
    DDS_1_Start();                 // initialise and enable
    Freq  = DDS_1_GetFrequency();
    Phase = DDS_1_GetPhase();
    
    Filter_Start();
    
    ADC1_Start();
    ADC1_StartConvert();//only for free-running ADC

    DMA_Config();//User-implemented function to set-up DMA
    
    isrReport_StartEx(IRQ_InterruptReport);
    ADC1_IRQ_StartEx(ADC1_IRQ_myInterrupt); // average ADC output using IIR filter
    isrDFB_StartEx(DFB_DataReady);            // average DFB output using IIR filter
}


int main()
{
    
    const uint16 offset =32000;

    
    Initialize();
    
    
    for(;;) //forever
    {
        
        if (Decoder_1_PositionChanged)
        {
            //BlinkLED1();                  // debug..
            switch(Encode_Type)
            {
                case ENCODE_FREQ: {       
                    UpdateDDSfrequency();
                    break;
                };
                
                case ENCODE_PHASE: {
                    UpdateDDSphase();
                    break;
                };             
            }
        }
        
        
        if (Decoder_1_BtnPressed)
        {
            //BlinkLED1();                  // debug..
            Encode_Type = (Encode_Type==ENCODE_FREQ)? ENCODE_PHASE : ENCODE_FREQ;
        }
        
        
        //ADC->
        if(isrADC_flag != 0) 
        {
            isrADC_flag = 0;
            ADC1_aver = IIR_Filter2(ADC1_result); // average ADC directly 
        }
        
        
        // Filter->
        if(isrDFB_flag != 0) // monitor for Filter interrupt
        {
            isrDFB_flag = 0;
            IIRout = IIR_Filter(FILTER_result); 
        }
        
        
        if(isrReport_flag != 0) // monitor for Report Timer interrupt
        {
            isrReport_flag = 0;
            //BlinkLED1();                  // debug..
            //FILTER_result=Filter_Read16(Filter_CHANNEL_A);
  
            //Multichart:  "RawData" (red) /  "Baseline" (blue) /  "Signal" (green)
            SendTxData(IIRout + offset,   FILTER_result + offset, ADC1_aver + offset);// send data to Multichart         
        }
        
        

    }  
    
} // main


//==============================================================================
// Update DDS frequency
//==============================================================================

void UpdateDDSfrequency()
{
    const double step = 100.0; // Hz, frequency increment
    
    double trial = (Decoder_1_Direction>0)? Freq + step : Freq - step;
    
    // can check for min/max limits here
    
    if (DDS_1_SetFrequency(trial))  //if success
    {
        Freq = trial;
    }
}

//==============================================================================
// Update DDS phase
//==============================================================================

void UpdateDDSphase()
{
    const uint8 step = 8; // increment phase (use 1,2,4,8.. e.g. 256%8=0)
    Phase += (Decoder_1_Direction>0)? step : -step;  
    DDS_1_SetPhase(Phase);
}


//==============================================================================
// IIR filter 1
//==============================================================================

int32 IIR_Filter(int32 input)
{
    int32 result;
    static int32 VAL_aver;        // IIR accumulator
    
    input <<= 8;
//    VAL_aver = VAL_aver + (((input-VAL_aver) >> 8) * 17);// factor 17/256
    VAL_aver = VAL_aver + (((input-VAL_aver) >> 8 >>2 ) );// factor 1/256 * 1/4 = 1/1024
    result = (VAL_aver>>8) + ((VAL_aver & 0x00000080) >> 7);//rounding
    return result;
}

//==============================================================================
// IIR filter 2
//==============================================================================

int32 IIR_Filter2(int32 input)
{
    int32 result;
    static int32 VAL_aver;        // IIR accumulator
    
    input <<= 8;
//    VAL_aver = VAL_aver + (((input-VAL_aver) >> 8) * 17);// factor 17/256
    VAL_aver = VAL_aver + (((input-VAL_aver) >> 8 >>2 ) );// factor 1/256 * 1/4 = 1/1024
    result = (VAL_aver>>8) + ((VAL_aver & 0x00000080) >> 7);//rounding
    return result;
}
